package com.lwby.aas.handle;

import cn.hutool.crypto.SecureUtil;
import com.lwby.aas.DeviceType;
import com.lwby.aas.po.AppChannel;
import com.lwby.aas.vo.DeliveryDeviceInfo;
import org.apache.commons.lang3.time.FastDateFormat;
import org.springframework.jdbc.core.JdbcTemplate;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.time.ZoneId;
import java.util.Date;
import java.util.TimeZone;
import java.util.concurrent.TimeUnit;

public class BaseHelper {
    public static final FastDateFormat DATE_TIME_FORMAT = FastDateFormat.getInstance("yyyy-MM-dd HH:mm:ss", TimeZone.getTimeZone(ZoneId.of("Asia/Shanghai")));

    public static final FastDateFormat DATE_BOTTOM_FORMAT = FastDateFormat.getInstance("yyyy_MM_dd", TimeZone.getTimeZone(ZoneId.of("Asia/Shanghai")));

    public String assembleKey(String deviceId, int platformId, DeviceType type) {
        if (DeviceType.IP == type) {
            return String.format("mck:%s:%d", SecureUtil.md5(deviceId), platformId);
        }
        return String.format("getClickByIdfaAndPlatformId_%s_%d", SecureUtil.md5(deviceId), platformId);
    }

    /**
     * 是否为当天时间
     *
     * @param date1
     * @return
     */
    public boolean isCurrentDayTime(Date date1) {
        Date date2 = new Date();
        return date1.getYear() == date2.getYear() && date1.getMonth() == date2.getMonth() && date1.getDate() == date2.getDate();
    }


    /**
     * 时间是否大于7天
     *
     * @param date
     * @return
     */
    public boolean isTimeGreaterThanSevenDays(Date date) {
        return isTimeGreaterThanDays(date,7);
    }


    /**
     * 时间是否大于指定天数
     *
     * @param date
     * @return
     */
    public boolean isTimeGreaterThanDays(Date date,long day) {
        return date.before(new Date(System.currentTimeMillis() - 60 * 60 * 24 * day * 1000));
    }

    /**
     * 计算注册时间
     * @return
     */
    public long calculateDaysBetweenDates(Date startDate, Date endDate) {
        return TimeUnit.DAYS.convert(Math.abs(endDate.getTime() - startDate.getTime()), TimeUnit.MILLISECONDS);
    }

    /**
     * 根据渠道ID和平台ID查询渠道配置信息
     * @param platformId
     * @param channelId
     * @return
     */
    public AppChannel getAppChannel(JdbcTemplate lwbyJdbcTemplate,int platformId,int channelId){
        return lwbyJdbcTemplate.queryForObject(String.format("select * from app_channel where platform_id=%d and channel_id=%d ORDER BY update_time DESC LIMIT 1",platformId,channelId),AppChannel.class);
    }


    /**
     * 获取明天零点时间戳
     *
     * @return 当天的零点时间戳
     */
    public long getTodayStartTime() {
        LocalDate tomorrow = LocalDate.now().plusDays(1);
        LocalDateTime tomorrowMidnight = LocalDateTime.of(tomorrow, LocalTime.MIDNIGHT);
        return tomorrowMidnight.atZone(ZoneId.of("Asia/Shanghai")).toInstant().toEpochMilli();
    }


    /**
     * REDIS set
     *
     * @param key
     * @param expires 过期时间
     * @param o
     */
    public void set(String key, int expires, Object o) {
    }

    /**
     * 为给定 key 设置生存时间，当 key 过期时(生存时间为 0 )，它会被自动删除。
     *
     * @param key     KEY值
     * @param seconds 过期时间，单位：秒
     */
    public void expire(String key, int seconds) {
    }

    public boolean exists(String key) {
        return false;
    }

    public <T> T get(Class<T> clazz, String key) {
        return null;
    }

    public Long incrby(String key, int increment, int expireSecond) {
        return -1l;
    }


    public static Long incrby(String key, int increment) {
        return -1l;
    }

    /**
     * 根据指定的userId和相对的激活日期来获取对应的激活次数
     *
     * @param userId           userId
     * @param lastestAliveDate 最后一次活跃时间
     * @return 对应的激活次数
     */
    public Integer getOldUserCountByUserIdAndlatestDate(String userId, Date lastestAliveDate) {
        return -1;
    }

    public DeliveryDeviceInfo getForRedis(String key) {
        return null;
    }


}
