package com.lwby.aas.handle;


import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONObject;
import com.lwby.aas.DeviceType;
import com.lwby.aas.enums.CallEnum;
import com.lwby.aas.util.BaseCallHelper;
import com.lwby.aas.vo.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.ApplicationContext;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

@Slf4j
public class TestHandle extends BaseHelper{
    List<AttributionHandle> handleList = new ArrayList<>();

    @Resource
    ApplicationContext applicationContext;

    @Resource
    BaseCallHelper baseCallHelper;

    @Value("${platformPrizeExpire:{}}")
    private String platformPrizeExpire;

    @PostConstruct
    public void load(){
        handleList = applicationContext.getBeansOfType(AttributionHandle.class).values().stream()
                .sorted(Comparator.comparingInt(AttributionHandle::order).reversed())
                .collect(Collectors.toList());
    }

    public void init(){
        String msg = "{\"bookStoreEvent\":1,\"clientInfo\":{\"channel\":220605094,\"clientIp\":\"171.14.28.126\",\"dID\":\"aad1da5913749ee5\",\"ddid\":\"null\",\"firm\":\"HUAWEI\",\"fixVersion\":12,\"mainVersion\":2,\"oaid\":\"f243f86a-8887-44df-92f3-7efed504b6f7\",\"os\":\"0\",\"phoneModel\":\"NOH-AN00\",\"pkv\":1,\"platformGroupId\":6,\"platformId\":6,\"pm\":\"NOH-AN00\",\"screenSize\":\"1152*2256\",\"sessionid\":\"wnL6tSDnvCjwnLtNJRDK299Z_jQEkUqSj\",\"signVersion\":2,\"subVersion\":27,\"systemVersion\":\"12\",\"ua\":\"Mozilla/5.0 (Linux; Android 12; NOH-AN00 Build/HUAWEINOH-AN00; wv) AppleWebKit/537.36 (KHTML, like Gecko) Version/4.0 Chrome/99.0.4844.88 Mobile Safari/537.36\",\"user\":{\"career\":\"\",\"channel\":220605094,\"deviceId\":\"aad1da5913749ee5\",\"experience\":1,\"gender\":\"M\",\"headImage\":\"http://cdn.ibreader.com/group1/M00/56/07/rBH0olunjiKAJCYCAAAQJEqQEcM308.png\",\"id\":133842748,\"isKaiqiUser\":false,\"isMajia\":false,\"isPrivate\":false,\"lastLogin\":1705980630000,\"level\":0,\"mainversion\":2,\"nickname\":\"书友742606\",\"platformId\":6,\"registrationDate\":1705980630000,\"subversion\":27,\"userStatus\":0,\"username\":\"bduXujqDcJbCZD\"},\"version\":\"6.2.27.12.220605094\",\"visitor\":\"wnL6tSDnvCjwnLtNJRDK299Z_jQEkUqSj\",\"xClient\":\"dID=aad1da5913749ee5;os=0;firm=HUAWEI;webVersion=new;version=6.2.27.12.220605094;username=wnL6tSDnvCjwnLtNJRDK299Z_jQEkUqSj;ddid=null;sv=12;pm=NOH-AN00;ss=1152*2256;signVersion=2;androidosv=31;oaid=f243f86a-8887-44df-92f3-7efed504b6f7;pkv=1;\"},\"createTime\":1705980632408,\"extraData\":{},\"id\":\"fd17b7d0-2ec2-4392-8980-170dbdf36e69\"}";

        BookStoreEvent event = JSONObject.parseObject(msg, BookStoreEvent.class);
        ClientInfo clientInfo = event.getClientInfo();

        //获取用户注册时间
        UserProfile userProfile = clientInfo.getUser();
        if (userProfile == null || userProfile.getRegistrationDate() == null) {
            log.error("ClientInfo.userProfile 对像为[{}],BookStoreEvent报文[{}]、对像[{}]",userProfile, msg, event);
            return;
        }

        //ANDROID deviceId
        String deviceId = clientInfo.getDID();
        //平台ID
        Integer platformId = clientInfo.getPlatformId();
        //设备ID类型
        DeviceType deviceType = null;
        //设备ID
        String deviceIdKey = null;
        //VO对像
        DeliveryDeviceInfo deliveryDeviceInfo = null;

        //匹配设备ID
        for(DeviceType type:DeviceType.values()){
            deviceIdKey = type.getDeviceId(clientInfo);
            if(null != deviceIdKey && null != (deliveryDeviceInfo = getForRedis(assembleKey(deviceIdKey,platformId,type)))){
                deviceType = type;
                clientInfo.setOaid(deliveryDeviceInfo.getOaid());   //回补OAID
                break;
            }
        }

        if(DeviceType.IP == deviceType){
            //TODO；只发BI 不上报逻辑
            baseCallHelper.summaryByUserChannel(clientInfo,CallEnum.IP_CALLBACK.getValue(),deliveryDeviceInfo);
            return;
        }

        Action action = new Action(clientInfo,deliveryDeviceInfo);
        for(AttributionHandle handle:handleList){
            if(handle.handle(action) != Flow.NEXT){
                break;
            }
        }
    }



    public static void main(String[] args) {
        new TestHandle().init();
    }
}
