package com.lwby.aas.handle.impl;

import com.lwby.aas.enums.CallEnum;
import com.lwby.aas.handle.AttributionHandle;
import com.lwby.aas.handle.AttributionType;
import com.lwby.aas.handle.BaseHelper;
import com.lwby.aas.handle.Flow;
import com.lwby.aas.po.AppChannel;
import com.lwby.aas.service.ICallbackService;
import com.lwby.aas.util.BaseCallHelper;
import com.lwby.aas.vo.Action;
import com.lwby.aas.vo.ClientInfo;
import com.lwby.aas.vo.DeliveryDeviceInfo;
import org.apache.commons.lang3.StringUtils;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.Date;
import java.util.concurrent.ThreadLocalRandom;


@Component
public class ChannelAttributionHandle extends BaseHelper implements AttributionHandle {

    @Resource
    JdbcTemplate marketingJdbcTemplate;

    @Resource
    JdbcTemplate lwbyJdbcTemplate;

    @Resource
    ICallbackService callbackService;

    @Resource
    BaseCallHelper baseCallHelper;

    @Override
    public int order() {
        return 0;
    }

    @Override
    public Flow handle(Action action) {
        return handle0(action, AttributionType.CHANNEL);
    }

    public Flow handle0(Action action,AttributionType type) {

        String dateStr = DATE_BOTTOM_FORMAT.format(new Date());
        String isDeducOneByTodayKey = String.format("v_deduc_%d_%d_%d_%s",action.getDeviceId(),action.getPlatformId(),action.getMedia(),dateStr);

        //是否是当天注册用户
        boolean isNewUser = isCurrentDayTime(action.getRegistrationDate());

        //获取满意配置
        AppChannel appChannel = getAppChannel(lwbyJdbcTemplate,action.getPlatformId(), action.getChannelId());

        //TODO；应该是等于空 或 = 100，如果空怎么办
        if (appChannel == null || appChannel.getSprDedu() == 100) {
            //TODO;回传
            callbackService.registerCallback(action);
            return Flow.END;
        }

        //如果是20  就是回传 20
        Integer sprDedu = appChannel.getSprDedu();

        //总数
        String channelTotal = String.format("%s_total_%d_%d_%d_%s",type.getName(), action.getPlatformId(), type.select(action.getChannelId(),action.getPlanId()), sprDedu, dateStr);
        //回传
        String channelCallback = String.format("%s_callback_%d_%d_%d_%s",type.getName(), action.getPlatformId(), type.select(action.getChannelId(),action.getPlanId()), sprDedu, dateStr);

        double percent = sprDedu.doubleValue();

        long channelTotalCount = incrby(channelTotal, 0, 60 * 60 * 24);
        long channelCallbackCount = incrby(channelCallback, 0, 60 * 60 * 24);

        //TODO;判断1-30天内是否活跃老用户,如果是则 100% 扣量；
        if(!isNewUser && isAliveByDay(action.getUserId(),30)){
            incrby(channelTotal, 1);
            //TODO;扣量
            baseCallHelper.summaryByUserChannel(action.getClientInfo(), CallEnum.OLDUSER_DEDUCTION_CALLBACK.getValue(), action.getDeliveryDeviceInfo());
            //每天扣量过一次，不在重复走
            set(isDeducOneByTodayKey,60 * 60 * 24,1);
            return Flow.END;
        }

        //首次随机
        if(0 == channelTotalCount){
            incrby(channelTotal, 1);
            if (ThreadLocalRandom.current().nextInt(1, 3) == 1) {
                //回传个数 ++
                incrby(channelCallback, 1);
                //TODO;回传事件
                callbackService.registerCallback(action);
                return Flow.END;
            }
        }else{
            //计算%
            BigDecimal divide = new BigDecimal(channelCallbackCount).divide(new BigDecimal(channelTotalCount == 0 ? 1 : channelTotalCount), 4, RoundingMode.HALF_UP);
            //总数 ++
            incrby(channelTotal, 1);
            // 扣量
            if (divide.compareTo(new BigDecimal(percent).setScale(4, RoundingMode.HALF_UP)) <= 0) {
                //TODO;回传事件
                incrby(channelCallback, 1);
                callbackService.registerCallback(action);
                return Flow.END;
            }
        }

        //TODO;扣量
        baseCallHelper.summaryByUserChannel(action.getClientInfo(), CallEnum.NORMAL_DEDUCTION_CALLBACK.getValue(), action.getDeliveryDeviceInfo());
        //每天扣量过一次，不在重复走
        set(isDeducOneByTodayKey,60 * 60 * 24,1);
        return Flow.END;
    }

    /**
     * 用户在指定天数内是否有活跃
     * @param userId
     * @param day
     * @return
     */
    public boolean isAliveByDay(long userId, int day){
        return marketingJdbcTemplate.queryForMap(String.format("select 1 from lwby_marketing_growth.alive_olduser where user_id ='%d' and last_alive_date  >= CURDATE() - INTERVAL %d DAY",userId,day)).size() > 0;
    }
}