package com.lwby.aas.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.lwby.aas.enums.MediaEnum;
import com.lwby.aas.factory.MediaActiveFactory;
import com.lwby.aas.service.ICallbackService;
import com.lwby.aas.service.IMediaActiveService;
import com.lwby.aas.vo.Action;
import com.lwby.aas.vo.ClientInfo;
import com.lwby.aas.vo.DeliveryDeviceInfo;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;

/**
 * @author songxinyu
 * @version CallbackServiceImpl.java, v 0.1 2024年02月23日 16:34 songxinyu Exp $
 */
@Slf4j
@Service
public class CallbackServiceImpl implements ICallbackService {

    @Resource
    MediaActiveFactory mediaActiveFactory;

    public final static Integer CALL_BACK_SUCCESS = 0;

    @Override
    @Async
    public void registerCallback(Action action) {

        ClientInfo clientInfo = action.getClientInfo();
        DeliveryDeviceInfo deliveryDeviceInfo = action.getDeliveryDeviceInfo();

        IMediaActiveService mediaActiveService = mediaActiveFactory.getMediaActiveService(MediaEnum.getByMediaName(action.getMedia()));

        Integer code = -1;

        if(StringUtils.isNotEmpty(clientInfo.getOs())){
            switch (clientInfo.getOs()){
                case "0":
                    code = mediaActiveService.androidActive(clientInfo,deliveryDeviceInfo);
                    break;
                case "1":
                    code = mediaActiveService.iosActive(clientInfo,deliveryDeviceInfo);
                    break;
                default:
                    code = mediaActiveService.androidActive(clientInfo,deliveryDeviceInfo);
                    break;
            }
        }else{
            code = mediaActiveService.androidActive(clientInfo,deliveryDeviceInfo);
        }

        if (!CALL_BACK_SUCCESS.equals(code)) {
            log.error("CallbackServiceImpl registerCallback result code error,code={},clientInfo={}",code, JSONObject.toJSONString(clientInfo));
            return;
        }
    }
}
