package com.lwby.aas.service.impl.gdt;

import cn.hutool.crypto.SecureUtil;
import cn.hutool.http.HttpUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.lwby.aas.enums.CallEnum;
import com.lwby.aas.enums.MediaEnum;
import com.lwby.aas.handle.BaseHelper;
import com.lwby.aas.service.IMediaActiveService;
import com.lwby.aas.service.impl.gdt.dto.Action;
import com.lwby.aas.service.impl.gdt.dto.GdtAttributeRequest;
import com.lwby.aas.service.impl.gdt.dto.UserId;
import com.lwby.aas.util.BaseCallHelper;
import com.lwby.aas.vo.ClientInfo;
import com.lwby.aas.vo.DeliveryDeviceInfo;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * @author songxinyu
 * @version GdtActiveImpl.java, v 0.1 2024年02月26日 10:25 songxinyu Exp $
 */
@Service
@Slf4j
public class GdtActiveImpl extends BaseHelper implements IMediaActiveService {

    @Resource
    BaseCallHelper baseCallHelper;

    public static final String ATTRIBUTE_EVENTTYPE = "ACTIVATE_APP";

    @Override
    public Integer androidActive(ClientInfo clientInfo, DeliveryDeviceInfo deliveryDeviceInfo) {
        Integer resultCode = -1;
        String result = "";
        StringBuilder sb = new StringBuilder(deliveryDeviceInfo.getCallback_url());
        long actionTime = System.currentTimeMillis() / 1000;

        UserId ui = UserId.builder()
                .ip(clientInfo.getClientIp())
                .oaid(clientInfo.getOaid())
                .hash_oaid(SecureUtil.md5(clientInfo.getOaid()))
                .hash_imei(clientInfo.getImei())
                .hash_android_id(clientInfo.getAndroidid())
                .user_agent(clientInfo.getUa())
                .build();

        Action action = Action.builder().action_time(actionTime)
                .user_id(ui)
                .action_type(ATTRIBUTE_EVENTTYPE).build();

        List actions = new ArrayList();
        actions.add(action);

        GdtAttributeRequest dto = GdtAttributeRequest.builder()
                .actions(actions)
                .build();
        String userAction = JSONObject.toJSONString(dto);
        try {
            result = HttpUtil.post(sb.toString(), userAction);
            resultCode = (Integer) JSON.parseObject(result).get("code");
            if(resultCode == 0){
                baseCallHelper.summaryByUserChannel(clientInfo, CallEnum.ACTIVE_CALLBACK.getValue(), deliveryDeviceInfo);
                log.info("GdtActiveImpl.androidActive : {}",JSONObject.toJSONString(deliveryDeviceInfo));
                String dateStr = DATE_BOTTOM_FORMAT.format(new Date());
                String isActiveOneByTodayKey = String.format("v_dec_%d_%d_%d_%s",clientInfo.getDID(),clientInfo.getPlatformId(),deliveryDeviceInfo.getMedia(),dateStr);
                //按天激活过一次，就不在上报
                set(isActiveOneByTodayKey, 60 * 60 * 24 ,1);
            }
        } catch (Exception e) {
            log.error("GdtActiveImpl.androidActivefive callback_param :{},userAction :{},result :{}" , deliveryDeviceInfo.getCallback_param(),JSONObject.toJSONString(userAction),result,e);
        }
        return null;
    }

    @Override
    public Integer iosActive(ClientInfo clientInfo, DeliveryDeviceInfo deliveryDeviceInfo) {
        return null;
    }

    @Override
    public MediaEnum getMediaEnum() {
        return MediaEnum.GDT;
    }
}
