package com.lwby.aas.service.impl.jrtt;

import cn.hutool.http.HttpUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.lwby.aas.enums.CallEnum;
import com.lwby.aas.enums.MediaEnum;
import com.lwby.aas.handle.BaseHelper;
import com.lwby.aas.service.IMediaActiveService;
import com.lwby.aas.service.impl.jrtt.dto.JrttAttributeRequest;
import com.lwby.aas.util.BaseCallHelper;
import com.lwby.aas.vo.ClientInfo;
import com.lwby.aas.vo.DeliveryDeviceInfo;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Date;

/**
 * @author songxinyu
 * @version JrttActiveImpl.java, v 0.1 2024年02月23日 16:42 songxinyu Exp $
 */
@Service
@Slf4j
public class JrttActiveImpl extends BaseHelper implements IMediaActiveService {

    public static final String ATTRIBUTE_URL = "https://analytics.oceanengine.com/api/v2/conversion";

    public static final String ATTRIBUTE_EVENTTYPE = "active";

    @Resource
    BaseCallHelper baseCallHelper;

    @Override
    public Integer androidActive(ClientInfo clientInfo, DeliveryDeviceInfo deliveryDeviceInfo) {
        Integer resultCode = -1;
        String result = "";
        StringBuilder sb = new StringBuilder(ATTRIBUTE_URL);
        JrttAttributeRequest.Device device = new JrttAttributeRequest.Device();
        device.setPlatform("android");
        device.setImei(clientInfo.getImei());
        device.setOaid(clientInfo.getOaid());
        JrttAttributeRequest.Ad ad = new JrttAttributeRequest.Ad();
        ad.setCallback(deliveryDeviceInfo.getCallback_param());
        JrttAttributeRequest.Context context = new JrttAttributeRequest.Context();
        context.setAd(ad);
        context.setDevice(device);

        JrttAttributeRequest request = JrttAttributeRequest.builder().context(context).event_type(ATTRIBUTE_EVENTTYPE).timestamp(
                System.currentTimeMillis()).build();
        String userJson = JSONObject.toJSONString(request);
        try {
            result = HttpUtil.post(sb.toString(), userJson);
            resultCode = (Integer) JSON.parseObject(result).get("code");
            log.info("JrttActiveImpl.androidActivefive callback_param :{},userJson :{},result :{}" , deliveryDeviceInfo.getCallback_param(),userJson,result);

            if(resultCode == 0){
                baseCallHelper.summaryByUserChannel(clientInfo, CallEnum.ACTIVE_CALLBACK.getValue(), deliveryDeviceInfo);
                log.info("JrttActiveImpl.androidActive : {}",JSONObject.toJSONString(deliveryDeviceInfo));
                String dateStr = DATE_BOTTOM_FORMAT.format(new Date());
                String isActiveOneByTodayKey = String.format("v_dec_%d_%d_%d_%s",clientInfo.getDID(),clientInfo.getPlatformId(),deliveryDeviceInfo.getMedia(),dateStr);
                //按天激活过一次，就不在上报
                set(isActiveOneByTodayKey, 60 * 60 * 24 ,1);
            }
        } catch (Exception e) {
            log.error("JrttActiveImpl.androidActivefive callback_param :{},userJson :{},result :{}" , deliveryDeviceInfo.getCallback_param(),JSONObject.toJSONString(userJson),result,e);
        }
        return resultCode;
    }

    @Override
    public Integer iosActive(ClientInfo clientInfo, DeliveryDeviceInfo deliveryDeviceInfo) {
        return -1;
    }

    @Override
    public MediaEnum getMediaEnum() {
        return MediaEnum.TOU_TIAO;
    }
}
