package com.lwby.aas.util;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.lwby.aas.enums.StoreEnum;
import com.lwby.aas.handle.BaseHelper;
import com.lwby.aas.vo.ClientInfo;
import com.lwby.aas.vo.DeliveryDeviceInfo;
import lombok.extern.slf4j.Slf4j;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.kafka.core.KafkaTemplate;
import org.springframework.stereotype.Component;
import org.springframework.util.concurrent.FailureCallback;
import org.springframework.util.concurrent.ListenableFuture;
import org.springframework.util.concurrent.SuccessCallback;

import javax.annotation.Resource;
import java.util.Arrays;
import java.util.List;

/**
 * @author songxinyu
 * @version BaseCallHelper.java, v 0.1 2024年02月23日 11:04 songxinyu Exp $
 */
@Slf4j
@Component
public class BaseCallHelper extends BaseHelper {

    @Resource
    RedisTemplate redisTemplate;

    @Resource
    private KafkaTemplate<String, String> kafkaTemplate;

    public void callbackToStore(ClientInfo clientInfo, Integer type) {
        String deviceId = clientInfo.getDID();
        Integer platformId = clientInfo.getPlatformId();
        //商店用户是否在该平台近30天访问的用户，是，不发给BI消息，否，发给BI消息
        String aliveDeviceKey = BaseCacheUtils.buildKey("market:activeness", 30, platformId, deviceId);
        if (!redisTemplate.hasKey(aliveDeviceKey)) {
            //发送给BI消息
            summaryByUserChannel(clientInfo, type,null);
            if (log.isInfoEnabled()) {
                log.info(
                        "deliveryDeviceInfo is null, send message, deviceId={}, platformId={}, aliveDeviceKey={}",
                        deviceId, platformId,aliveDeviceKey);
            }
        } else {
            //不发送给BI消息
            if (log.isInfoEnabled()) {
                log.info(
                        "deliveryDeviceInfo is null, no send message, deviceId={}, platformId={}, aliveDeviceKey={}",
                        deviceId, platformId,aliveDeviceKey);
            }
        }
    }

    /**
     * 根据用户注册渠道归因
     * @param clientInfo
     * @param type 区分不回传和从商店发大数据消息标识
     */
    public void summaryByUserChannel(ClientInfo clientInfo, Integer type, DeliveryDeviceInfo deliveryDeviceInfo) {
        Integer channel = clientInfo.getChannel();
        String channelStr = String.valueOf(channel);
        String channelPrefix = channelStr.substring(0, 2);
        final String media = StoreEnum.getStoreDescByChannelPre(channelPrefix);
        String device_id = clientInfo.getDID();
        String prizeAssembleKey = "c:ouser:";
        if (deliveryDeviceInfo == null) {
            //商店的
            deliveryDeviceInfo = new DeliveryDeviceInfo();
            deliveryDeviceInfo.setOs(clientInfo.getOs());
            //type如果为2，代表扣量，设置返回BI is_call字段为4，否则还是设置返回BI is_call字段为3
            deliveryDeviceInfo.setIs_call(type);
            deliveryDeviceInfo.setDevice_id(device_id);
            deliveryDeviceInfo.setMedia(media);
            deliveryDeviceInfo.setAd_plan_id("0");
            deliveryDeviceInfo.setAd_group_id("0");
            deliveryDeviceInfo.setAd_creative_id("0");
            deliveryDeviceInfo.setActive_time(System.currentTimeMillis());
            deliveryDeviceInfo.setChannel(String.valueOf(clientInfo.getChannel()));
            deliveryDeviceInfo.setPlatform_id(String.valueOf(clientInfo.getPlatformId()));
            //deliveryDeviceInfo.setOcpc_device_id(matchDeviceId);
            //deliveryDeviceInfo.setOcpc_device_id_type(deviceIdType);
            deliveryDeviceInfo.setDj_channel(channelStr);
            //deliveryDeviceInfo.setEncryption_type(encryptType);
            if (clientInfo.getUser() == null) {
                log.error("BaseCallHelper.summaryMediaBySource store clientInfo.getUser() is null : {}", JSONObject.toJSONString(clientInfo));
            } else {
                deliveryDeviceInfo.setUserId(clientInfo.getUser().getId());
                if (exists(prizeAssembleKey.concat(String.valueOf(clientInfo.getUser().getId())))) {
                    deliveryDeviceInfo.setDevice_status(2);
                    log.info("BaseCallHelper is set status,clinetinfo={}, deliveryDeviceInfo={}",JSONObject.toJSONString(clientInfo), JSON.toJSONString(deliveryDeviceInfo));
                } else {
                    deliveryDeviceInfo.setDevice_status(0);
                    log.info("BaseCallHelper is not set status,clinetinfo={}, deliveryDeviceInfo={}",JSONObject.toJSONString(clientInfo), JSON.toJSONString(deliveryDeviceInfo));
                }
            }
        } else {
            //三个不回传条件的,扣量的
            //deliveryDeviceInfo.setOcpc_device_id(matchDeviceId);
            //deliveryDeviceInfo.setOcpc_device_id_type(deviceIdType);
            //deliveryDeviceInfo.setEncryption_type(encryptType);
            deliveryDeviceInfo.setOs(clientInfo.getOs());
            deliveryDeviceInfo.setIs_call(type);
            deliveryDeviceInfo.setDevice_id(device_id);
            deliveryDeviceInfo.setActive_time(System.currentTimeMillis());
            deliveryDeviceInfo.setChannel(String.valueOf(clientInfo.getChannel()));
            deliveryDeviceInfo.setUserId(clientInfo.getUser().getId());
            if (clientInfo.getUser() == null) {
                log.error("BaseCallHelper.summaryMediaBySource clientInfo.getUser() is null : {}", JSONObject.toJSONString(clientInfo));
            } else {
                deliveryDeviceInfo.setUserId(clientInfo.getUser().getId());
                if (exists(prizeAssembleKey.concat(String.valueOf(clientInfo.getUser().getId())))) {
                    deliveryDeviceInfo.setDevice_status(2);
                    log.info("BaseCallHelper.dedu is set status,clinetinfo={}, deliveryDeviceInfo={}",JSONObject.toJSONString(clientInfo), JSON.toJSONString(deliveryDeviceInfo));
                } else {
                    deliveryDeviceInfo.setDevice_status(0);
                    log.info("BaseCallHelper.dedu is not set status,clinetinfo={}, deliveryDeviceInfo={}",JSONObject.toJSONString(clientInfo), JSON.toJSONString(deliveryDeviceInfo));
                }
            }
        }
        ListenableFuture active_result = kafkaTemplate.send("ocpc_result", JSONObject.toJSONString(deliveryDeviceInfo));
        active_result.addCallback(new SuccessCallback() {
            @Override
            public void onSuccess(Object o) {
                log.info("BaseCallHelper.summaryByUserChannel done,sendKafkaMessageType:{},归因 成功 :{} " ,type,o);
            }
        }, new FailureCallback() {
            @Override
            public void onFailure(Throwable throwable) {
                log.info("BaseCallHelper.summaryByUserChannel done,sendKafkaMessageType:{},归因出错,业务回滚或补偿 :{} " ,type,throwable);
            }
        });
        log.info("BaseCallHelper.summaryByUserChannel done,store_access: device_id:{},channel:{},sendKafkaMessageType:{}",device_id,channel,type);
    }
}
