package com.lwby.aas.util;

import lombok.extern.slf4j.Slf4j;
import org.rocksdb.Options;
import org.rocksdb.RocksDB;
import org.rocksdb.RocksDBException;

import java.io.File;
import java.util.Objects;

@Slf4j
public final class DataOperate {
    private RocksDB rocksDB;

    static {
        RocksDB.loadLibrary();
    }

    public DataOperate(String path) throws RocksDBException {
        File file = new File(path);
        if(!file.exists()){
            file.mkdirs();
        }

        Options options = new Options();
        options.setCreateIfMissing(true);
        rocksDB = RocksDB.open(options,path);
    }

    public boolean exist(String key) {
        try {
            return !Objects.isNull(rocksDB.get(key.getBytes()));
        } catch (RocksDBException e) {
            throw new RuntimeException(e);
        }
    }

    public void set(String key,byte[] value){
        try {
            rocksDB.put(key.getBytes(), value);
        } catch (RocksDBException e) {
            throw new RuntimeException(e);
        }
    }

    public byte[] get(String key){
        try {
             return rocksDB.get(key.getBytes());
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    public void setLong(String key,long value) {
        try {
            rocksDB.put(key.getBytes(), BytesUtils.getBytes(value));
        } catch (RocksDBException e) {
            throw new RuntimeException(e);
        }
    }

    public long getLong(String key) {
        try {
            byte[] b = rocksDB.get(key.getBytes());
            return b == null?0:BytesUtils.bytesToLong(b);
        } catch (RocksDBException e) {
            throw new RuntimeException(e);
        }
    }

    public void close(){
       rocksDB.close();
    }
}
