package com.lwby.aas.util;

/**
 * SESSION ID 生成器
 * @author martin.ad
 * @Description:
 * @date 创建时间： 2022/9/20 16:09
 */

import java.util.Arrays;
import java.util.Objects;

public class SessionIdGenerator {
    private final static char[] ENCODE_DIC = "wnL6tS1obdOmYV9rMGBkgI8RJvUp3H2xya-u5DENseP_zKAihlqQTZc4X0j7CfFW".toCharArray();

    private final static byte[] DECODE_DIC = new byte[256];

    private final static String UTF_8 = "UTF-8";

    private final static int SESSION_LENGTH = 33;

    private final static int LONG_ENCODE_LENGTH=11;

    static {
        Arrays.fill(DECODE_DIC, (byte) -1);
        for (int i = 0; i < ENCODE_DIC.length; i++) {
            DECODE_DIC[ENCODE_DIC[i]] = (byte) i;
        }
    }

    public static String encode(byte[] bytes) {
        if (bytes == null) {
            return "";
        }
        if (bytes.length == 0) {
            return new String(new char[] { ENCODE_DIC[0] });
        }
        int fullGroups = bytes.length / 3;
        int remainBytes = bytes.length % 3;
        char[] buf = new char[(bytes.length * 4 - 1) / 3 + 1];
        int i = 0;
        for (; i < fullGroups; i++) {
            int b0 = 255 & bytes[i * 3];
            int b1 = 255 & bytes[i * 3 + 1];
            int b2 = 255 & bytes[i * 3 + 2];
            int c0 = i * 4;
            int c1 = i * 4 + 1;
            int c2 = i * 4 + 2;
            int c3 = i * 4 + 3;
            buf[c0] = ENCODE_DIC[encrypt(c0, b0 >>> 2)];
            buf[c1] = ENCODE_DIC[encrypt(c1, ((3 & b0) << 4) | (b1 >>> 4))];
            buf[c2] = ENCODE_DIC[encrypt(c2, ((15 & b1) << 2) | (b2 >>> 6))];
            buf[c3] = ENCODE_DIC[encrypt(c3, 63 & b2)];
        }
        switch (remainBytes) {
            case 2:
                int b0 = 255 & bytes[i * 3];
                int b1 = 255 & bytes[i * 3 + 1];
                int c0 = i * 4;
                int c1 = i * 4 + 1;
                int c2 = i * 4 + 2;
                buf[c0] = ENCODE_DIC[encrypt(c0, b0 >>> 2)];
                buf[c1] = ENCODE_DIC[encrypt(c1, ((3 & b0) << 4) | (b1 >>> 4))];
                buf[c2] = ENCODE_DIC[encrypt(c2, ((15 & b1) << 2))];
                break;
            case 1:
                b0 = 255 & bytes[i * 3];
                c0 = i * 4;
                c1 = i * 4 + 1;
                buf[c0] = ENCODE_DIC[encrypt(c0, b0 >>> 2)];
                buf[c1] = ENCODE_DIC[encrypt(c1, ((3 & b0) << 4))];
                break;
        }
        return new String(buf);
    }

    public static byte[] decode(String str) {
        if (str == null || str.length() == 0) {
            return null;
        }
        if (str.length() == 1) {
            return new byte[0];
        }
        char[] chars = str.toCharArray();
        int fullGroups = chars.length / 4;
        int remainChars = chars.length % 4;
        byte[] ret = new byte[(chars.length - 1) * 3 / 4 + 1];
        int i = 0;
        for (; i < fullGroups; i++) {
            int c0 = i * 4;
            int c1 = i * 4 + 1;
            int c2 = i * 4 + 2;
            int c3 = i * 4 + 3;
            int b0 = decrypt(c0, DECODE_DIC[chars[c0]]);
            int b1 = decrypt(c1, DECODE_DIC[chars[c1]]);
            int b2 = decrypt(c2, DECODE_DIC[chars[c2]]);
            int b3 = decrypt(c3, DECODE_DIC[chars[c3]]);
            ret[i * 3] = (byte) ((b0 << 2) | (b1 >>> 4));
            ret[i * 3 + 1] = (byte) ((b1 << 4) | (b2 >>> 2));
            ret[i * 3 + 2] = (byte) ((b2 << 6) | b3);
        }
        switch (remainChars) {
            case 3:
                int c0 = i * 4;
                int c1 = i * 4 + 1;
                int c2 = i * 4 + 2;
                int b0 = decrypt(c0, DECODE_DIC[chars[c0]]);
                int b1 = decrypt(c1, DECODE_DIC[chars[c1]]);
                int b2 = decrypt(c2, DECODE_DIC[chars[c2]]);
                ret[i * 3] = (byte) ((b0 << 2) | (b1 >>> 4));
                ret[i * 3 + 1] = (byte) ((b1 << 4) | (b2 >>> 2));
                break;
            case 2:
                c0 = i * 4;
                c1 = i * 4 + 1;
                b0 = decrypt(c0, DECODE_DIC[chars[c0]]);
                b1 = decrypt(c1, DECODE_DIC[chars[c1]]);
                ret[i * 3] = (byte) ((b0 << 2) | (b1 >>> 4));
                break;
        }
        return ret;
    }


    private static int encrypt(int charIndex, int tableIndex) {
        return (charIndex + tableIndex) % 64;
    }

    private static int decrypt(int charIndex, int crptyIndex) {
        int r = crptyIndex - charIndex % 64;
        if (r < 0) {
            r += 64;
        }
        return r;
    }

    public static byte[] getBytes(long data) {
        byte[] bytes = new byte[8];
        bytes[0] = (byte) ((data >> 56) & 0xff);
        bytes[1] = (byte) ((data >> 48) & 0xff);
        bytes[2] = (byte) ((data >> 40) & 0xff);
        bytes[3] = (byte) ((data >> 32) & 0xff);
        bytes[4] = (byte) ((data >> 24) & 0xff);
        bytes[5] = (byte) ((data >> 16) & 0xff);
        bytes[6] = (byte) ((data >> 8) & 0xff);
        bytes[7] = (byte) (data & 0xff);
        return bytes;
    }

    public static long getLong(byte[] bytes) {
        return (0xff00000000000000L & ((long) bytes[0] << 56)
                | (0xff000000000000L & ((long) bytes[1] << 48))
                | (0xff0000000000L & ((long) bytes[2] << 40))
                | (0xff00000000L & ((long) bytes[3] << 32))
                | (0xff000000L & ((long) bytes[4] << 24))
                | (0xff0000L & ((long) bytes[5] << 16))
                | (0xff00L & ((long) bytes[6] << 8)) | (0xffL & (long) bytes[7]));
    }

    public static String generateId(long args0,long args1,long args2){
        return new StringBuffer(encode(getBytes(args0))).
                            append(encode(getBytes(args1))).
                            append(encode(getBytes(args2))).
                            toString();
    }

    public static Long[] decryptParams(String sessoinId){
        Objects.requireNonNull("sessoinId");

        if(sessoinId.length() == SESSION_LENGTH){
            Long[] params = new Long[3];
            for(int i=0;i<params.length;i++){
                params[i] = getLong(decode(sessoinId.substring(i * LONG_ENCODE_LENGTH, (i+1)*LONG_ENCODE_LENGTH)));
            }
            return params;
        }
        return null;
    }

    public static long decryptParam(String sessoinId,int index){
        Objects.requireNonNull("sessoinId");

        if(sessoinId.length() == 33){
            return getLong(decode(sessoinId.substring(index*11, (index + 1) * 11)));
        }

        return 0;
    }
}
